'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createPropertyExpression = function createPropertyExpression(identifier, props) {
  var entries = _lodash2.default.uniq(props).sort().map(function (prop) {
    return t.stringLiteral(prop);
  });

  var left = t.memberExpression(t.identifier(identifier), t.identifier('handledProps'));
  var right = t.arrayExpression(entries);

  return t.expressionStatement(t.assignmentExpression('=', left, right));
};

var findTarget = function findTarget(path) {
  if (t.isArrowFunctionExpression(path) || t.isFunctionExpression(path)) {
    var declarationPath = path.findParent(function (parentPath) {
      return t.isVariableDeclaration(parentPath);
    });

    return findTarget(declarationPath);
  }

  if (t.isExportDeclaration(path.parent)) return path.findParent(function (parentPath) {
    return t.isExportDeclaration(parentPath);
  });
  if (t.isExpression(path)) return path.findParent(function (parentPath) {
    return t.isDeclaration(parentPath);
  });

  return path;
};

var insertAfterPath = function insertAfterPath(path, expression) {
  return findTarget(path).insertAfter(expression);
};

var insertEntries = function insertEntries(entries) {
  return entries.forEach(function (_ref) {
    var identifier = _ref.identifier,
        path = _ref.path,
        props = _ref.props;

    insertAfterPath(path, createPropertyExpression(identifier, props));
  });
};

exports.default = insertEntries;